#include "BoundingBox.h"
#include <GL/glut.h>

BoundingBox::BoundingBox(const Vector3f *pos, const Vector3f& min, 
						 const Vector3f& max) : BoundingVolume(pos)
{
	minExtent = min;
	maxExtent = max;
}

bool BoundingBox::inside(const Vector3f& point) const
{
    Vector3f absMin = *position + minExtent;
	Vector3f absMax = *position + maxExtent;
	
	return ((absMin.x <= point.x && point.x <= absMax.x) &&
		    (absMin.y <= point.y && point.y <= absMax.y) &&
		    (absMin.z <= point.z && point.z <= absMax.z));
}

bool BoundingBox::positiveSide(const Plane& plane) const
{
	float radius = (maxExtent - minExtent).length() / 2.0f;

	return (dotProduct(plane.normal, *position) + plane.distance < -radius);
}

void BoundingBox::draw() const
{
	glPushMatrix();
	glPushAttrib(GL_COLOR_BUFFER_BIT);
	glColor3f(1.0f, 0.0f, 0.0f);
	glTranslatef(position->x, position->y, position->z);
   
	// Front face
	glBegin(GL_LINE_LOOP);
	glVertex3f(minExtent.x, minExtent.y, minExtent.z);
	glVertex3f(maxExtent.x, minExtent.y, minExtent.z);
	glVertex3f(maxExtent.x, maxExtent.y, minExtent.z);
	glVertex3f(minExtent.x, maxExtent.y, minExtent.z);
	glEnd();

	// Back face
	glBegin(GL_LINE_LOOP);
	glVertex3f(minExtent.x, minExtent.y, maxExtent.z);
	glVertex3f(minExtent.x, maxExtent.y, maxExtent.z);
	glVertex3f(maxExtent.x, maxExtent.y, maxExtent.z);
	glVertex3f(maxExtent.x, minExtent.y, maxExtent.z);
	glEnd();

	// connecting lines
	glBegin(GL_LINES);

	glVertex3f(minExtent.x, minExtent.y, minExtent.z);
	glVertex3f(minExtent.x, minExtent.y, maxExtent.z);
	   
	glVertex3f(minExtent.x, maxExtent.y, minExtent.z);
	glVertex3f(minExtent.x, maxExtent.y, maxExtent.z);
	
	glVertex3f(maxExtent.x, minExtent.y, minExtent.z);
	glVertex3f(maxExtent.x, minExtent.y, maxExtent.z);

	glVertex3f(maxExtent.x, maxExtent.y, minExtent.z);
	glVertex3f(maxExtent.x, maxExtent.y, maxExtent.z);

	glEnd();
	glPopAttrib();
	glPopMatrix();
}