#include "BoundingSphere.h"
#include "math_util.h"
#include <GL/glut.h>

bool BoundingSphere::inside(const Vector3f& point) const
{
	return (position->sqDistance(point) <= radius * radius);
}

bool BoundingSphere::positiveSide(const Plane& plane) const
{ 
	return (dotProduct(plane.normal, *position) + plane.distance < -radius); 
}

bool BoundingSphere::intersects(const Vector3f& oldPosition, const Vector3f& newPosition) const
{
	Vector3f closestPoint = closestPointOnLine(oldPosition, newPosition, *position);

	return inside(closestPoint);
}

bool BoundingSphere::intersects(const BoundingVolume* volume) const
{
	const BoundingSphere *sphere = dynamic_cast<const BoundingSphere*>(volume);

	if (sphere == 0) return false;

	float sqDist = (radius + sphere->radius) * (radius + sphere->radius);

	return (position->sqDistance(sphere->getPosition()) < sqDist);
}

float BoundingSphere::distance(const Vector3f& pos) const
{ 
	return position->distance(pos); 
}

void BoundingSphere::draw() const
{
	glPushMatrix();
	glPushAttrib(GL_COLOR_BUFFER_BIT);
	glColor4f(1.0f, 0.5f, 0.0f, 0.3f);
	glTranslatef(position->x, position->y, position->z);
	glEnable(GL_CULL_FACE);
	glutSolidSphere(radius, 10, 10);
	glDisable(GL_CULL_FACE);
	glPopAttrib();
	glPopMatrix();
}
