#ifndef BOUNDINGVOLUME_H
#define BOUNDINGVOLUME_H

#include "Vector3.h"
#include "Frustrum.h"

class BoundingVolume
{
protected:
	// A pointer to the position of the object that has this volume
	const Vector3f *position;

	BoundingVolume() { visible = false; };
public:
	bool visible;

	BoundingVolume(const Vector3f *position) 
	{ this->position = position; visible = false; }

	const Vector3f& getPosition() const { return *position; }
	void setPosition(const Vector3f* pos) 
	{ position = pos; }

	void toggleVisible() { visible = !visible; }

	// Test if the point is inside this volume
	virtual bool inside(const Vector3f& point) const = 0;

	// Test if the volume is on the positive side of the plane
	virtual bool positiveSide(const Plane& plane) const = 0;

	// Find if this volume intersects the line segment
	// defined by a projectile's oldPosition and newPosition
	virtual bool intersects(const Vector3f& oldPosition, const Vector3f& newPosition) const = 0;

	// Find if this volume intersects the other Bounding volume
	virtual bool intersects(const BoundingVolume* volume) const = 0;

	// Draw this volume
	virtual void draw() const = 0;
};

#endif