#include <iostream>
#include <vector>
#include "ClientSocket.h"
#include "Message.h"

//constructors and destructor
ClientSocket::ClientSocket()
{
  winsockInit = false;
}

ClientSocket::ClientSocket(char *host, int port)
{
  if (!(initWinsock() && openSocket(host, port)))
	  std::cout << "Error in ClientSocket()" << std::endl;
}

ClientSocket::~ClientSocket()
{
	closeSocket();
  if(winsockInit) deinitWinsock();
}

//returns true if it was able to initialize winsock
bool ClientSocket::initWinsock()
{
  int error = WSAStartup (0x0202, &wsaDat);

  if (error)
  {
	  std::cout << "Error in initWinsock()" << std::endl;
    winsockInit = false;
    return false;
  }
  if (wsaDat.wVersion != 0x0202)
  { // wrong WinSock version!
	  std::cout << "Error in version" << std::endl;
    WSACleanup (); // unload ws2_32.dll
	winsockInit = false;
    return false;
  }
  winsockInit = true;
  return true;
}

void ClientSocket::deinitWinsock()
{
	WSACleanup();
	winsockInit = false;
}

bool ClientSocket::openSocket(char *host, int port)
{
  theSocket = socket(AF_INET, SOCK_STREAM, 0);

  u_long addr = inet_addr(host);
  //if host string isn't an ip try do a DNS lookup
  if (addr == INADDR_NONE) 
  {
    hostent* HE = gethostbyname(host);
	//unable to parse hostname return false
    if (HE == 0) 
	{
		std::cout << "Error in connect(): HE=0" << std::endl;
		return false;
    }
	//get the most valid ip
    addr = *((u_long*)HE->h_addr_list[0]);
  }


  sockaddr_in serverAddr;

  serverAddr.sin_family = AF_INET;           // address family Internet
  serverAddr.sin_port = htons (port);        // set servers port number
  serverAddr.sin_addr.s_addr = addr;         // set servers IP

  if (connect(theSocket, (LPSOCKADDR)&serverAddr, sizeof(serverAddr)) == SOCKET_ERROR)
  { 
	  std::cout << "Error in connect():" << WSAGetLastError() << std::endl;
	  return false;
  }

  return true;
}

void ClientSocket::closeSocket()
{
	closesocket(theSocket);
}

bool ClientSocket::sendData(char* buffer, int size)
{
  int sendVal = send(theSocket, buffer, size, 0);
  if(sendVal == SOCKET_ERROR) return false;
  else return true;
}

bool ClientSocket::receiveData(char* buffer, int size)
{
  fd_set input_set, exc_set; // Create Input and Error sets for the select function
  int s, // This is Used for the Select command to store the value it gets back
  nfds; // Used for Compatability

  timeval timeout;

  timeout.tv_sec=0; // Used for Timeout
  timeout.tv_usec=0;

  //Set up the input,and exception sets for select().
  FD_ZERO(&input_set);
  FD_ZERO(&exc_set);
  nfds = 0;

  FD_SET(theSocket,&input_set);
  FD_SET(theSocket,&exc_set);


  // The Actual Select Function sees if data coming in and stores it in s
  s = select(nfds,&input_set,NULL,&exc_set,&timeout);
	if(s > 0)
	{
		recv(theSocket, buffer, size, 0);
		return true;
	}
	else return false;
}

bool ClientSocket::receiveData(char *buffer)
{
	return receiveData(buffer, Max_Buffer);
}

bool ClientSocket::readLine(char *line)
{
	std::vector<char> theVector;
	char buffer;

	while (true) 
	{
		bool validRecv = receiveData(&buffer, 1);

		if(!validRecv) return false;

		if (buffer == '\n') 
		{
			theVector.push_back(buffer);

			for (u_int f = 0; f < theVector.size(); f++) 
				line[f] = theVector[f];

			return true;
		}
		else 
			theVector.push_back(buffer);
	}
}

bool ClientSocket::isWinsockInit()
{
  return winsockInit;
}
