//#include "Vector3.h"
#include "Matrix.h"
#include "Frustrum.h"
#include "math_util.h"
#include "BoundingVolume.h"
#include <GL/glut.h>

void Frustrum::recalcPlanes()
{
	Matrix matProj, matModelView;

	glGetFloatv(GL_PROJECTION_MATRIX, (float*) &matProj);
	glGetFloatv(GL_MODELVIEW_MATRIX, (float*) &matModelView);

	Matrix matrix = matProj * matModelView;

	planes[leftPlane].set(matrix.m41 + matrix.m11, matrix.m42 + matrix.m12, 
					 matrix.m43 + matrix.m13, matrix.m44 + matrix.m14);

	planes[rightPlane].set(matrix.m41 - matrix.m11, matrix.m42 - matrix.m12, 
					  matrix.m43 - matrix.m13, matrix.m44 - matrix.m14);

	planes[bottomPlane].set(matrix.m41 + matrix.m21, matrix.m42 + matrix.m22, 
					   matrix.m43 + matrix.m23, matrix.m44 + matrix.m24);

	planes[topPlane].set(matrix.m41 - matrix.m21, matrix.m42 - matrix.m22, 
					matrix.m43 - matrix.m23, matrix.m44 - matrix.m24);

	planes[nearPlane].set(matrix.m41 + matrix.m31, matrix.m42 + matrix.m32, 
					 matrix.m43 + matrix.m33, matrix.m44 + matrix.m34);

	planes[farPlane].set(matrix.m41 - matrix.m31, matrix.m42 - matrix.m32, 
					matrix.m43 - matrix.m33, matrix.m44 - matrix.m34);

	volumesCulled = 0;
}

bool Frustrum::inside(const BoundingVolume* bVolume) const
{
	if (enabled)
		for (int i = 0; i < NoPlanes; i++)
			if (bVolume->positiveSide(planes[i]))
			{
				volumesCulled++;
				return false;
			}

	return true;
}
