#ifndef GEOMETRY_H
#define GEOMETRY_H

class BoundingVolume;

class Geometry
{
protected:
	// The OpenGL list numbers
	int listNo;

	// Multiple objects can refer to the same geometry
	// we need some way to keep track of references
	mutable int refCount;

	void setupList();
	virtual void setupVertices() = 0;

public:
	Geometry() : refCount(0) { }
	virtual ~Geometry() { }

	// Multiple objects can refer to the same geometry
	Geometry* alloc() { refCount++; return this; }
	void dealloc() const { --refCount; }

	// Since the geometry knows its extents, it can best
	// decide its bounding volume
	virtual BoundingVolume* getBoundingVolume() const = 0;

	// OpenGL display lists are faster than immediate mode
	// If we have a problem with this, a subclass could override this
	virtual void draw() const;
};

#endif