#include "LocalPlayer.h"
#include "BoundingVolume.h"
#include "Geometry.h"
#include "Weapon.h"
#include <GL/glut.h>

#ifdef MULTI
#include "Message.h"
#include "ClientSocket.h"
extern ClientSocket connection;
#endif

LocalPlayer::LocalPlayer(Geometry *g, const Vector3f& startPos, const Vector3f& startDir) 
	: Player(g, startPos, startDir)
{
	strafeVector.set(0.0f, 0.0f, 0.0f);
}

void LocalPlayer::update(float time)
{
	Player::update(time);

	if (!alive) return;
	position += strafeVector * velocity * time;
	strafeVector.set(0.0f, 0.0f, 0.0f);
}

void LocalPlayer::keyboardInput(unsigned char key)
{
	if (!alive) return;

	switch (toupper(key))
	{
		case 'W':
			incSpeed();
			break;
		case 'S':
			decSpeed();
			break;
		case 'A':
			strafe(-20.0f);
			break;
		case 'D':
			strafe(+20.0f);
			break;
		case 'Q':
			camera->rotate(-rollAmt, rotAxis.z);
			roll(-rollAmt);
			break;
		case 'E':
			camera->rotate(+rollAmt, rotAxis.z);
			roll(+rollAmt);
			break;
	}
}

void LocalPlayer::mouseInput(float deltaX, float deltaY)
{
	if (!alive) return;

	if (fabs(deltaX) > 0.001f)
	{
		yaw(deltaX);
		camera->rotate(deltaX, rotAxis.y);
	}
	
	if (fabs(deltaY) > 0.001f)
	{
		pitch(deltaY);
		camera->rotate(deltaY, rotAxis.x);
	}

	assert(equals(dotProduct(rotAxis.x, rotAxis.y), 0.0f));
	assert(equals(dotProduct(rotAxis.x, rotAxis.z), 0.0f));
	assert(equals(dotProduct(rotAxis.y, rotAxis.z), 0.0f));
}

void LocalPlayer::strafe(float dir)
{
	strafeVector = rotAxis.x * dir;
	strafeVector.normalize();
}

void LocalPlayer::draw() const
{
	if (!visible || !alive)	return;

	glPushMatrix();
	glLoadIdentity();

	// These are specific to the model we are using right now
	glTranslatef(0.0f, -0.4f, -0.8f);
	glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
  
	glColor3f(color.x, color.y, color.z);
	geometry->draw();

	glPopMatrix();
}


void LocalPlayer::drawCrossHair() const
{
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_TEXTURE_2D);
	Vector3f pos(position+rotAxis.z*10.0f);
	glBegin(GL_LINES);
	glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
	glVertex3f(pos.x+1.0f, pos.y, pos.z);
	glVertex3f(pos.x-1.0f, pos.y, pos.z);
	glVertex3f(pos.x, pos.y+1.0f, pos.z);
	glVertex3f(pos.x, pos.y-1.0f, pos.z);
	glVertex3f(pos.x, pos.y, pos.z+1.0f);
	glVertex3f(pos.x, pos.y, pos.z-1.0f);
	glEnd();
}

void LocalPlayer::hit(const Projectile* projectile, const Vector3f& location)
{
	Player::hit(projectile, location);
	if (shield < 0)
		die();
}

void LocalPlayer::die()
{
	Player::die();

	//std::cout << "Shield is below 0!" << std::endl;
	#ifdef MULTI
	char buffer[Max_Buffer];
	Message::PlayerDie(buffer);
	connection.sendData(buffer, (int) strlen(buffer));
	#endif
}
