//Message is a glorified parser used to store and recieve game state
//information to and from a character buffer

#include "Message.h"
#include <string.h>
#include <stdio.h>

//Messages-----------------------------------------------------------------------------------------
void Message::AllMessage(char *externalBuf, Vector3f pos, Vector3f dir, Vector3f up, float vel, Quaternion qua)
{
	sprintf(externalBuf, "all: %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
		      pos.x, pos.y, pos.z, dir.x, dir.y, dir.z, up.x, up.y, up.z,
					vel, qua.x, qua.y, qua.z, qua.w);
}

void Message::PositionMessage(char *externalBuf, float posX, float posY, float posZ)
{
	sprintf(externalBuf, "pos: %f, %f, %f\n", posX, posY, posZ);
}
void Message::PositionMessage(char *externalBuf, Vector3f pos)
{
	sprintf(externalBuf, "pos: %f, %f, %f\n", pos.x, pos.y, pos.z);
}

void Message::BulletMessage(char *externalBuf, int type, Vector3f pos, Vector3f dir, float vel)
{
	sprintf(externalBuf, "bul: %d %f %f %f %f %f %f %f\n", type, pos.x, pos.y, pos.z, dir.x, dir.y, dir.z, vel);
}

void Message::DirectionMessage(char *externalBuf, float dirX, float dirY, float dirZ)
{
	sprintf(externalBuf, "dir: %f, %f, %f\n", dirX, dirY, dirZ);
}
void Message::DirectionMessage(char *externalBuf, Vector3f dir)
{
	sprintf(externalBuf, "dir: %f, %f, %f\n", dir.x, dir.y, dir.z);
}

void Message::UpMessage(char *externalBuf, float upX, float upY, float upZ)
{
	sprintf(externalBuf, "up: %f, %f, %f\n", upX, upY, upZ);
}
void Message::UpMessage(char *externalBuf, Vector3f up)
{
	sprintf(externalBuf, "up: %f, %f, %f\n", up.x, up.y, up.z);
}

void Message::RotationMessage(char *externalBuf, float rotX, float rotY, float rotZ, float rotW)
{
	sprintf(externalBuf, "rot: %f, %f, %f, %f\n", rotX, rotY, rotZ, rotW);
}
void Message::RotationMessage(char *externalBuf, Vector3f rot, float rotW)
{
	sprintf(externalBuf, "rot: %f, %f, %f, %f\n", rot.x, rot.y, rot.z, rotW);
}
void Message::RotationMessage(char *externalBuf, Quaternion qua)
{
	sprintf(externalBuf, "rot: %f, %f, %f, %f\n", qua.x, qua.y, qua.z, qua.w);
}
	
void Message::VelocityMessage(char *externalBuf, float vel)
{
	sprintf(externalBuf, "vel: %f\n", vel);
}

void Message::CollisionMessage(char *externalBuf, char *type1, int id1, char *type2, int id2)
{
	sprintf(externalBuf, "col: %s %d and %s %d\n", type1, id1, type2, id2);
}

void Message::StateMessage(char *externalBuf, float shield, bool destroyed)
{
	sprintf(externalBuf, "shield: %f destroyed: %d\n", shield, destroyed);
}

void Message::PlayerInit(char *externalBuf, int ship, int team, Vector3f pos)
{
	sprintf(externalBuf, "init: %d %d %f %f %f\n", ship, team, pos.x, pos.y, pos.z);
}

void Message::PlayerDie(char *externalBuf)
{
	sprintf(externalBuf, "killed\n");
}

void Message::PlayerQuit(char *externalBuf)
{
	sprintf(externalBuf, "quit\n");
}

void Message::GameStart(char *externalBuf)
{
	sprintf(externalBuf, "gameStart\n");
}

void Message::GameOver(char *externalBuf)
{
	sprintf(externalBuf, "endGame\n");
}

void Message::YouMessage(char *externalBuf, int playerNum)
{
	sprintf(externalBuf, "you: %d\n", playerNum);
}

//Parses-------------------------------------------------------------------------------------------
bool Message::ParseAll(char *externalBuf, Vector3f *pos, Vector3f *dir, Vector3f *up, float *vel, Quaternion *qua)
{
	int numScanned = sscanf(externalBuf, "all: %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
		               &(pos->x), &(pos->y), &(pos->z), &(dir->x), &(dir->y), &(dir->z), &(up->x), &(up->y), &(up->z),
									 vel, &(qua->x), &(qua->y), &(qua->z), &(qua->w));
	if( numScanned != 14 ) return false;
	else return true;
}

bool Message::ParsePosition(char *externalBuf, float *posX, float *posY, float *posZ)
{
	int numScanned = sscanf(externalBuf, "pos: %f, %f, %f\n", posX, posY, posZ);
	if( numScanned != 3 ) return false;
	else return true;
}
bool Message::ParsePosition(char *externalBuf, Vector3f *pos)
{
	float x, y, z;
	int numScanned = sscanf(externalBuf, "pos: %f, %f, %f\n", &x, &y, &z);
	pos->set(x, y, z);
	if( numScanned != 3 ) return false;
	else return true;
}

bool Message::ParseBullet(char *externalBuf, int *type, Vector3f *pos, Vector3f *dir, float *vel)
{
	int numScanned = sscanf(externalBuf, "bul: %d %f %f %f %f %f %f %f\n", type, &(pos->x), &(pos->y), &(pos->z),
		                      &(dir->x), &(dir->y), &(dir->z), vel);
	if( numScanned != 8 ) return false;
	else return true;
}

bool Message::ParseDirection(char *externalBuf, float *dirX, float *dirY, float *dirZ)
{
	int numScanned = sscanf(externalBuf, "dir: %f, %f, %f\n", dirX, dirY, dirZ);
	if( numScanned != 3 ) return false;
	else return true;
}
bool Message::ParseDirection(char *externalBuf, Vector3f *dir)
{
	int numScanned = sscanf(externalBuf, "dir: %f, %f, %f\n", &(dir->x), &(dir->y), &(dir->z));
	if( numScanned != 3 ) return false;
	else return true;
}

bool Message::ParseUp(char *externalBuf, float *upX, float *upY, float *upZ)
{
	int numScanned = sscanf(externalBuf, "up: %f, %f, %f\n", upX, upY, upZ);
	if( numScanned != 3 ) return false;
	else return true;
}
bool Message::ParseUp(char *externalBuf, Vector3f *up)
{
	int numScanned = sscanf(externalBuf, "up: %f, %f, %f\n", &(up->x), &(up->y), &(up->z));
	if( numScanned != 3 ) return false;
	else return true;
}

bool Message::ParseRotation(char *externalBuf, float *rotX, float *rotY, float *rotZ, float *rotW)
{
	int numScanned = sscanf(externalBuf, "rot: %f, %f, %f, %f\n", rotX, rotY, rotZ, rotW);
	if( numScanned != 4 ) return false;
	else return true;
}
bool Message::ParseRotation(char *externalBuf, Vector3f *rot, float *rotW)
{
	int numScanned = sscanf(externalBuf, "rot: %f, %f, %f, %f\n", &(rot->x), &(rot->y), &(rot->z), rotW);
	if( numScanned != 4 ) return false;
	else return true;
}
bool Message::ParseRotation(char *externalBuf, Quaternion *qua)
{
	int numScanned = sscanf(externalBuf, "rot: %f, %f, %f, %f\n", &(qua->x), &(qua->y), &(qua->z), &(qua->w));
	if( numScanned != 4 ) return false;
	else return true;
}

bool Message::ParseVelocity(char *externalBuf, float *vel)
{
	int numScanned = sscanf(externalBuf, "vel: %f\n", vel);
	if( numScanned != 1 ) return false;
	else return true;
}

bool Message::ParseCollision(char *externalBuf, char *type1, int *id1, char *type2, int *id2)
{
	int numScanned = sscanf(externalBuf, "col: %s %d and %s %d\n", type1, id1, type2, id2);
	if( numScanned != 4 ) return false;
	else return true;
}

bool Message::ParseState(char *externalBuf, float *shield, bool *destroyed)
{
	int numScanned = sscanf(externalBuf, "shield: %f destroyed: %d\n", shield, destroyed);
	if( numScanned != 2 ) return false;
	else return true;
}

bool Message::ParseInit(char *externalBuf, int *ship, int *team, Vector3f *pos)
{
	int numScanned = sscanf(externalBuf, "init: %d %d %f %f %f\n", ship, team, &(pos->x), &(pos->y), &(pos->z));
	if( numScanned != 5 ) return false;
	else return true;
}


bool Message::ParseYou(char *externalBuf, int *playerNum)
{
	int numScanned = sscanf(externalBuf, "you: %d\n", playerNum);
	if( numScanned != 1 ) return false;
	else return false;
}

//remove the client number from the string
bool Message::StripClientNum(char *externalBuf)
{
  char *poundLoc;

	poundLoc = strchr(externalBuf, 35); //find the location of the pound symbol
	
  if(poundLoc == NULL) return false;

	strcpy(externalBuf, poundLoc + 1);
	return true;
}

//return the client number of the message negative value if there is an error
int Message::getClientNum(char *externalBuf)
{
	int clientNum;
	int numScanned = sscanf(externalBuf, "%d", &clientNum);
	if( numScanned != 1 ) return false;
	else return true;
}
