#ifndef MESSAGE_H
#define MESSAGE_H

#include "Vector3.h"
#include "Quaternion.h"

#define Max_Buffer 256

class Message
{
public:
	//------------------------------------------------------------------------------------------------
	//Create Appropriate Message from given variables and store in externalBuf
  static void AllMessage(char *externalBuf, Vector3f pos, Vector3f dir, Vector3f up, float vel, Quaternion qua);

	static void PositionMessage(char *externalBuf, float posX, float posY, float posZ); //pos:
	static void PositionMessage(char *externalBuf, Vector3f pos);

	static void BulletMessage(char *externalBuf, int type, Vector3f pos, Vector3f dir, float vel); //bul:

	static void DirectionMessage(char *externalBuf, float dirX, float dirY, float dirZ); //dir:
	static void DirectionMessage(char *externalBuf, Vector3f dir);

	static void UpMessage(char *externalBuf, float upX, float upY, float upZ);  //up:
	static void UpMessage(char *externalBuf, Vector3f up);

	static void RotationMessage(char *externalBuf, float rotX, float rotY, float rotZ, float rotW); //rot:
	static void RotationMessage(char *externalBuf, Vector3f rot, float rotW);
	static void RotationMessage(char *externalBuf, Quaternion qua);

	static void VelocityMessage(char *externalBuf, float vel); //vel:

	static void CollisionMessage(char *externalBuf, char *type1, int id1, char *type2, int id2); //col:

	static void StateMessage(char *externalBuf, float shield, bool destroyed); //shield:

	static void PlayerInit(char *externalBuf, int ship, int team, Vector3f pos); //init
	static void PlayerDie(char *externalBuf); //killed
	static void PlayerQuit(char *externalBuf); //quit

	static void GameStart(char *externalBuf); //gameStart
	static void GameOver(char *externalBuf); //endGame

	static void YouMessage(char *externalBuf, int playerNum);
  //------------------------------------------------------------------------------------------------
	//Parse information back out from a Message that is stored in externalBuf return false if it was
	//unsuccessful
  static bool ParseAll(char *externalBuf, Vector3f *pos, Vector3f *dir, Vector3f *up, float *vel, Quaternion *qua);
  
	static bool ParsePosition(char *externalBuf, float *posX, float *posY, float *posZ);
	static bool ParsePosition(char *externalBuf, Vector3f *pos);

	static bool ParseBullet(char *externalBuf, int *type, Vector3f *pos, Vector3f *dir, float *vel);

	static bool ParseDirection(char *externalBuf, float *dirX, float *dirY, float *dirZ);
	static bool ParseDirection(char *externalBuf, Vector3f *dir);

	static bool ParseUp(char *externalBuf, float *upX, float *upY, float *upZ);
	static bool ParseUp(char *externalBuf, Vector3f *up);

	static bool ParseRotation(char *externalBuf, float *rotX, float *rotY, float *rotZ, float *rotW);
	static bool ParseRotation(char *externalBuf, Vector3f *rot, float *rotW);
	static bool ParseRotation(char *externalBuf, Quaternion *qua);

	static bool ParseVelocity(char *externalBuf, float *vel);

	static bool ParseCollision(char *externalBuf, char *type1, int *id1, char *type2, int *id2);

	static bool ParseState(char *externalBuf, float *shield, bool *destroyed);

	static bool ParseInit(char *externalBuf, int *ship, int *team, Vector3f *pos);

	static bool ParseYou(char *externalBuf, int *playerNum);

	//-----------------------------------------------------------------------------------------------
	//Utility Methods

	//remove the client number from the string
	static bool StripClientNum(char *externalBuf);

	//return the client number of the message negative value if there is an error
	static int getClientNum(char *externalBuf);
};
#endif