#include ".\model3ds.h"
#include "BoundingSphere.h"

Model3ds::Model3ds(void) {
   uniform_texture = -1;
}

Model3ds::Model3ds(char* filename, float y_deg, float scle, unsigned int tex) {
   model.numOfMaterials = 0;
   model.numOfObjects = 0;

   max_x = max_y = max_z = -999999;
   min_x = min_y = min_z = 999999;
   totalVertices = 0;
   center.x = center.y = center.z = 0.0f;
   
   load3dsFile(filename);

   //determine the models max extent
   max_extent = max_x - min_x;
   if(max_y - min_y > max_extent)
      max_extent = max_y - min_y;
   if(max_z - min_z > max_extent)
      max_extent = max_z - min_z;

   center.scale(1.0f / (float)totalVertices);

   y_rotation = y_deg;
   scle > 0 ? scale = scle : scale = 1.0f;

   //determine radius
   Vector3f currentVertex;
   radius = 0.0f;
   float dist;
   for(int i = 0; i < model.numOfObjects; i++) {
      for(int j = 0; j < model.pObject[i].numOfVerts; j++) {
         currentVertex.x = model.pObject[i].pVerts[j].x;
         currentVertex.y = model.pObject[i].pVerts[j].y;
         currentVertex.z = model.pObject[i].pVerts[j].z;

         dist = center.distance(currentVertex);

         if(dist > radius)
            radius = dist;
      }
   }

   uniform_texture = tex;

   glEnable(GL_TEXTURE_2D);
   setupList();
}

Model3ds::~Model3ds(void) {
}

BoundingVolume* Model3ds::getBoundingVolume() const
{ return new BoundingSphere(radius * scale / max_extent); }

void Model3ds::load3dsFile(char* filename) {
	load.Import3DS(&model, filename);			// Load our .3DS file into our model structure
   
   //disables any texture/materials
   while(model.pMaterials.size() > 0)
      model.pMaterials.pop_back();
   model.numOfMaterials = 0;

	// Depending on how many textures we found, load each one (Assuming .BMP)
	// If you want to load other files than bitmaps, you will need to adjust CreateTexture().
	// Below, we go through all of the materials and check if they have a texture map to load.
	// Otherwise, the material just holds the color information and we don't need to load a texture.

	// Go through all the materials
	for(int i = 0; i < model.numOfMaterials; i++)
	{
		// Check to see if there is a file name to load in this material
		if(strlen(model.pMaterials[i].strFile) > 0)
		{
			// Use the name of the texture file to load the bitmap, with a texture ID (i).
			// We pass in our global texture array, the name of the texture, and an ID to reference it.	

         textures[i] = TextureLoading::LoadTexture(model.pMaterials[i].strFile);			
		}

		// Set the texture ID for this material
		model.pMaterials[i].texureId = i;
	}

   for(int i = 0; i < model.numOfObjects; i++)
   {
      totalVertices += model.pObject[i].numOfVerts;
      for(int j = 0; j < model.pObject[i].numOfVerts; j++)
      {
         center.x += model.pObject[i].pVerts[j].x;
         center.y += model.pObject[i].pVerts[j].y;
         center.z += model.pObject[i].pVerts[j].z;

         if(model.pObject[i].pVerts[j].x > max_x)
            max_x = model.pObject[i].pVerts[j].x;
         if(model.pObject[i].pVerts[j].x < min_x)
            min_x = model.pObject[i].pVerts[j].x;
         if(model.pObject[i].pVerts[j].y > max_y)
            max_y = model.pObject[i].pVerts[j].y;
         if(model.pObject[i].pVerts[j].y < min_y)
            min_y = model.pObject[i].pVerts[j].y;
         if(model.pObject[i].pVerts[j].z > max_z)
            max_z = model.pObject[i].pVerts[j].z;
         if(model.pObject[i].pVerts[j].z < min_z)
            min_z = model.pObject[i].pVerts[j].z;
      }
   }
}

void Model3ds::setupVertices() {
   int index;

	//makes the ship model face forward
	glRotatef(y_rotation, 0, 1.0f, 0);						// Rotate the object around the Y-Axis
	glScalef(scale/max_extent, scale/max_extent, scale/max_extent);
	glTranslatef(-center.x, -center.y, -center.z);

   if(uniform_texture != -1) {
      for(int i = 0; i < model.numOfObjects; i++) {
		   // Make sure we have valid objects just in case. (size() is in the vector class)
		   if(model.pObject.size() <= 0) break;

		   // Get the current object that we are displaying
		   t3DObject *pObject = &model.pObject[i];
		   glBegin(GL_TRIANGLES);
         //glBindTexture(GL_TEXTURE_2D, uniform_texture);
         for(int j = 0; j < pObject->numOfFaces; j++) {
		   	index = pObject->pFaces[j].vertIndex[0];
            glTexCoord2f(0.0, 0.0);
				glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);
				glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
			   index = pObject->pFaces[j].vertIndex[1];
            glTexCoord2f(0.0, 1.0);
            glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);
				glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
			   index = pObject->pFaces[j].vertIndex[2];
            glTexCoord2f(1.0, 0.0);
				glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);
				glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
		   }
		   glEnd();
      }
   }
   else {
      for(int i = 0; i < model.numOfObjects; i++) {
		   // Make sure we have valid objects just in case. (size() is in the vector class)
		   if(model.pObject.size() <= 0) break;

		   // Get the current object that we are displaying
		   t3DObject *pObject = &model.pObject[i];
		   glBegin(GL_TRIANGLES);
         for(int j = 0; j < pObject->numOfFaces; j++) {
		   	for(int whichVertex = 0; whichVertex < 3; whichVertex++) {
				   	int index = pObject->pFaces[j].vertIndex[whichVertex];
				   	glNormal3f(pObject->pNormals[ index ].x, pObject->pNormals[ index ].y, pObject->pNormals[ index ].z);
				   	glVertex3f(pObject->pVerts[ index ].x, pObject->pVerts[ index ].y, pObject->pVerts[ index ].z);
			   }
		   }
		   glEnd();
      }
   }
}

void Model3ds::draw() const {

   if (uniform_texture != -1) {
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, uniform_texture);
   }
   glCallList(listNo);
}
