#include "BoundingVolume.h"
#include "Frustrum.h"
#include "Geometry.h"
#include "Object3d.h"
#include <windows.h>
#include <GL/gl.h>

ObjectCollection Object3d::objects;

Object3d::Object3d(Geometry *g, const Vector3f& startPos) : Object(g, startPos) 
{ 
	culled = false;
	color.set(1.0f, 1.0f, 1.0f);

	if (g)
	{
		bVolume = g->getBoundingVolume();
		bVolume->setPosition(&position);
	}
	else
		bVolume = 0;

	objects.push_back(this);
}

Object3d::~Object3d()
{
	objects.remove(this);

	if (bVolume) 
	{
		delete bVolume;
		bVolume = 0;
	}
}

void Object3d::setBoundVolume(BoundingVolume *vol)
{ 
	if (bVolume) delete bVolume;
	bVolume = vol; 
}

void Object3d::toggleBVolumeVisible() 
{ 
	bVolume->toggleVisible(); 
}

void Object3d::testFrustrum(const Frustrum& frustrum)
{ 
	culled = !(frustrum.inside(bVolume));
}

void Object3d::draw() const
{ 
	if (culled) return;

	glColor3f(color.x, color.y, color.z);
	Object::draw();
}

void Object3d::drawBoundingVolume()
{
	if (visible && bVolume->visible) bVolume->draw();
}