#ifndef PLAYER_H
#define PLAYER_H

#include "Vector3.h"
#include "Object3d.h"
#include "Spacecraft.h"
#include "Weapon.h"
class Geometry;

const float rollAmt = 5.0f*PI/180.0f;

class Player : public Object3d
{
protected:	
	const static float MaxVelocity;
	const static float MaxEnergy;
	const static float MaxShield;
	const static float CollisionDamage;

	float shield;
	float energy;
	float velocity;
	float acceleration;
	float actualVelocity;

	float energyRegen;
	float shieldRegen;

	Vector3<Vector3f> rotAxis;

	Weapon *leftWeapon, *rightWeapon;

	Vector3f collForce;

	float currentTime;
	
	void setAxisVectors(const Vector3f& dir, const Vector3f& up);
public:
	bool alive;

	Player() : Object3d(0, Vector3f(0.0f, 0.0f, 0.0f)) { }
	Player(Geometry *g, const Vector3f& startPos, const Vector3f& startDir);
	~Player();

	void updateFromServer(char* buffer);
	
	void update(float time);
	void draw() const;
	virtual void drawBoundingVolume();
	virtual void hit(const Projectile* projectile, const Vector3f& location);
	virtual void die();

	void fireFirst();
	void fireSecond();
	void fireOther(int type);
	void setPrimaryWeapon(int type);
	void setSecondaryWeapon(int type);

	void pitch(float amount);
	void yaw(float amount);
	void roll(float dir);
	

	float getSpeed() const { return velocity; }
	float getActualSpeed() const { return actualVelocity; }
	float getShield() const { return shield; }
	float getEnergy() const { return energy; }
	const Vector3f& getDirection() const { return rotAxis.z; }
	const Vector3f& getUpvector() const { return rotAxis.y; }
};

#endif