#include "PlayerShield.h"
#include "Shader.h"
#include "math_util.h"
#include <GL/glut.h>

PlayerShield::PlayerShield(const Vector3f *pos, float radius, const char *filename) : BoundingSphere(pos, radius) 
{
	shader = new Shader(filename);

	timeLastHit = 0.0f;
}

PlayerShield::~PlayerShield()
{
	if (shader)
		delete shader;
}

void PlayerShield::drawShield(float time) const
{
	if (shader->enabled)
	{
		float spot = 1.0 + (time - timeLastHit) * 8.0f;
		if (equals(timeLastHit, 0.0f) || (spot > 9.0f)) return;

		// Hit Point
		shader->setVector4fParam(4, pointLastHit, 1.0f);
		// Material
		shader->setVector4fParam(5, 0.0f, 0.5f, 1.0f, 0.5f);
		// Shininess
		shader->setFloatParam(6, spot);

		glPushMatrix();
		glPushAttrib(GL_COLOR_BUFFER_BIT);
		glTranslatef(position->x, position->y, position->z);
		//glEnable(GL_CULL_FACE);

		shader->enable();
		glutSolidSphere(radius, 100, 100);
		shader->disable();
		
		//glDisable(GL_CULL_FACE);
		glPopAttrib();
		glPopMatrix();
	}
}
