#include "Object3d.h"
#include "Projectile.h"
#include "BoundingVolume.h"

std::list<Projectile*> Projectile::projectiles;

Projectile::Projectile(int texture, const Vector3f& startPos, const float size, const Object3d* player)
	: Object2d(texture, startPos, size)
{
	owner = player;
	life = 100; 
}

void Projectile::update(float time)
{ 
	Vector3f newPosition = position + velocity * time; 

	ObjectIter objIter = Object3d::objects.begin();
	ObjectIter objEndIter = Object3d::objects.end();
	const BoundingVolume* bVol;

	for (; objIter != objEndIter; objIter++)
	{
		bVol = (*objIter)->getBoundVolume();

		if ((owner != *objIter) && bVol->intersects(position, newPosition))
		{			
			Vector3f closest = closestPointOnLine(position, newPosition, bVol->getPosition());
			(*objIter)->hit(this, closest);
			life = 0;
			return;
		}
	}

	position = newPosition;
	life--;
}

void Projectile::addProj(Projectile* proj)
{
	projectiles.push_back(proj);
}

void Projectile::updateAll(float time)
{
	if (equals(time, 0.0f)) return;

	std::list<Projectile*>::iterator projIter = projectiles.begin();
	std::list<Projectile*>::iterator projEndIter = projectiles.end();

	while (projIter != projEndIter)
	{
		(*projIter)->update(time);

		if ((*projIter)->alive())
			projIter++;
		else
			projIter = projectiles.erase(projIter);
	}
}

void Projectile::drawAll()
{
	std::list<Projectile*>::const_iterator projIter = projectiles.begin();
	std::list<Projectile*>::const_iterator projEndIter = projectiles.end();

	for (; projIter != projEndIter; projIter++)
		(*projIter)->draw();
}