#include "Quaternion.h"
#include "Plane.h"

void Quaternion::set(const Vector3f& axis, const float angle)
{
	x = axis.x * sin(angle/2.0f);
	y = axis.y * sin(angle/2.0f);
	z = axis.z * sin(angle/2.0f);
	w = cos(angle/2.0f);

	normalize();
}

void Quaternion::set(const Vector3f& first, const Vector3f& second)
{
	Vector3f begin(first), end(second);
	begin.normalize(); end.normalize();

    float angle = acos(dotProduct(begin, end));
	Vector3f axis(0.0f, 0.0f, 0.0f);

	if (equals(angle, 0.0f)) 
		set(0.0f, 0.0f, 0.0f, 1.0f);
	else if (equals(angle, PI))
	{
		if (equals(first.y, 0.0f))
			set(Vector3f(0.0f, 1.0f, 0.0f), angle);
		else
		{
			while (equals(dotProduct(axis, first), 0.0f))
			{
				axis = Plane(begin).getRandomVector();
				axis.normalize();
			}

			set(axis, angle);
		}
	}
	else
	{
		crossProduct(begin, end, &axis);
		axis.normalize();
		set(axis, angle);
	}		
}

void Quaternion::set(const float pitch, const float yaw, const float roll)
{
	// calculate trig numbers
	//float cr = cos(roll/2.0f);
	//float cp = cos(pitch/2.0f);
	//float cy = cos(yaw/2.0f);

	//float sr = sin(roll/2.0f);
	//float sp = sin(pitch/2.0f);
	//float sy = sin(yaw/2.0f);

    //x = sr * cp * cy - cr * sp * sy;
	//y = cr * sp * cy + sr * cp * sy;
	//z = cr * cp * sy - sr * sp * cy;
	//w = cr * cp * cy + sr * sp * sy;

	Quaternion qx(sin(pitch/2.0f), 0.0f, 0.0f, cos(pitch/2.0f));
	Quaternion qy(0.0f, sin(yaw/2.0f), 0.0f, cos(yaw/2.0f));
	Quaternion qz(0.0f, 0.0f, sin(roll/2.0f), cos(roll/2.0f));

	*this = (qx * qy * qz);
}

float Quaternion::length() const
{ 
	return sqrt(x * x + y * y + z * z + w * w); 
}

Quaternion Quaternion::conjugate() const
{ 
	Quaternion result;

	result.x = -x;  
	result.y = -y;  
	result.z = -z; 
	result.w = w;

	return result;
}

void Quaternion::normalize()
{
	float len = length();

	assert (len > 0.0f);

	x /= len; y /= len;
	z /= len; w /= len;
}

void Quaternion::getAxisAngle(Vector3f &vec, float &angle) const
{
	float halfAngle = acos(w);
	float scale = sqrt(x*x + y*y + z*z);

	bound<float>(0.0f, halfAngle, PI);

	if (equals(scale, 0.0f))
	{
		angle = 0.0f;
		vec.set(0.0f, 1.0f, 0.0f);
	}
	else
	{
		angle = halfAngle * 2.0f;
		vec.set(x / scale, y / scale, z / scale);
		vec.normalize();
	}
}

std::ostream& operator<<(std::ostream& out, const Quaternion& quat)
{ out << "(" << quat.x << ", " << quat.y << ", " << quat.z << ", " << quat.w << ")"; return out; }
