#ifndef QUATERNION_H
#define QUATERNION_H

#include "math_util.h"

struct Matrix;

struct Quaternion
{
	float x, y, z, w;

	Quaternion() { }

	// Create a quaternion using the four values it stores
	Quaternion(const float ix, const float iy, const float iz, const float iw)
	{ set(ix, iy, iz, iw); }
	void set(const float ix, const float iy, const float iz, const float iw)
	{ x = ix; y = iy; z = iz; w = iw; }

	// Create a quaternion from a vector as [0, vec]
	Quaternion(const Vector3f& vec)
	{ x = vec.x; y = vec.y; z = vec.z; 	w = 0.0f; }

	// Make the quaternion represent a rotation of angle radians around axis
	Quaternion(const Vector3f& axis, const float angle)
	{ set(axis, angle); }
	void set(const Vector3f& axis, const float angle);

	void set(const Quaternion& quat)
	{ x = quat.x; y = quat.y; z = quat.z; w = quat.w; }

	// Make the quaternion represent the rotation required 
	// to make the first vector become the second
	void set(const Vector3f& first, const Vector3f& second);

	// Convert the Euler angles into a quaternion
	Quaternion(const float pitch, const float yaw, const float roll)
	{ set(pitch, yaw, roll); }
	void set(const float pitch, const float yaw, const float roll);

	// Normalize this quaternion
	void normalize();

	float length() const;	
	Quaternion conjugate() const;

	void getAxisAngle(Vector3f &vec, float &angle) const;
	//Matrix getRotationMatrix() const;

	void operator*=(const Quaternion& quat)
	{ *this = (quat * (*this)); }
};

std::ostream& operator<<(std::ostream& out, const Quaternion& quat);
#endif