//much of this code is from Richard Hallett's Tutorial on nonblocking servers, found on gamedev.net
#include <iostream>
#include <winsock.h>
#include <stdio.h>
#include <vector>
#include "..\Message.h"

using namespace std;

// Definitions-------------------------------------------------------------------------------------

#define PORT 12345            // The Port Number of the Server
#define Max_Connections 8   // The Maximum Number of Connections allowed to connect

#define YES 1  // Making 1 as YES, so its easier to understand our code.
#define NO  2  // Making 2 as NO, so its easier to understand our code.

struct CLIENT_TYP // This is the Structure that describes a Client.
{
  int InUse;                    // Is The Client Connected to Server
  SOCKET ClientSocket;          // A Socket for the client to sit on
  struct sockaddr_in clnt_addr; // Holds the address of the client
	int ship;
	int team;
	Vector3f pos;
};

// Globals-----------------------------------------------------------------------------------------

SOCKET ListeningSocket;  // This Socket Listens for Connections from the clients

struct sockaddr_in Server_Address; // This Structure Holds the address of the Server.

CLIENT_TYP Client[Max_Connections]; // This Sets up Client Structures for However
                                    // many we set the MAX to.

int ClientsConnected = 0; // How many clients are currently connected to the server

timeval timeout; // This is used for the select function its seeing
                 // how long it should wait before timing out.

// Functions---------------------------------------------------------------------------------------
//forward definition
char *readLine(SOCKET);


// Simple Error Message Code
bool Error(char *error)
{
  cout << error; "/n";
  return (true);
}

// Sets Up Server
void Setup()
{
  // Local Variables
  WSADATA wsaData;

  // Set up Address Structure
  Server_Address.sin_family = AF_INET;
  Server_Address.sin_addr.s_addr = INADDR_ANY;
  Server_Address.sin_port = htons(PORT);

  // Start Up Winsock
  if (WSAStartup(0x0202, &wsaData))
  {
    Error("Error while attempting to initialize WinSock");
  }

  // Create a Socket
  ListeningSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

  // Bind the Socket to Port
  bind(ListeningSocket, (struct sockaddr *)&Server_Address, sizeof(Server_Address));

  // Listen at the Binded Socket for Connections
  listen(ListeningSocket, SOMAXCONN);

  // Make all Clients Not In Use
  for (int i = 0; i < Max_Connections; i++)
  {
    Client[i].InUse = NO;
  }
} 

// Function That Disconnects a Specified Client
int Disconnect(int s)
{
  closesocket(Client[s].ClientSocket);
  Client[s].InUse = NO;
  ClientsConnected--;
	
  return(1);
}

// Shutdown the Entire Server
void Shutdown_Server()
{
  // Close The Socket that Listens for Connections
  closesocket(ListeningSocket);
	
  // Loop that closes down all the Client Connections
  for (int index = 0; index < Max_Connections, index++;)
  {
    Disconnect(index);
  }
	
  // Remove Winsock
  WSACleanup();
}

// Check For Connections
int Check_Connections()
{
  int s; // s is where the data is stored from the select function
  int nfds; // This is used for Compatibility
  fd_set conn; // Setup the read variable for the Select function
  int Client_Length = sizeof(Client->clnt_addr); // Store the length of the client address

  timeout.tv_sec=0; // Set the timeout to 0 for Non Blocking
  timeout.tv_usec=0;
	
  if (ClientsConnected < Max_Connections)
  {
    FD_ZERO(&conn); // Set the data in conn to nothing
    FD_SET(ListeningSocket, &conn); // Tell it to get the data from the Listening Socket
    // Up the nfds value by one, shouldnt be the same for each
    // client that connects for compatability reasons
    nfds=ListeningSocket+1;
    s = select(nfds, &conn, NULL, NULL, &timeout); // Is there any data coming in?

    if (s > 0) // Someone is trying to Connect
    {
      for (int index = 0; index < Max_Connections; index++)
      {
        if (Client[index].InUse == NO)
        {
          Client[index].ClientSocket = accept(ListeningSocket,
                                              (struct sockaddr *)&Client[index].clnt_addr,
                                              &Client_Length);

          // Client Connected
          // Store the Ip of the Client that Just Connected.
          char *ClientIp = inet_ntoa(Client[ClientsConnected].clnt_addr.sin_addr);
          cout << "Accepted Connection From: " << ClientIp << endl;

          // Now that a client has connected up the number of clients thats connected
          ClientsConnected++;

          // This Client is now in use.
          Client[index].InUse = YES;

          // Exit the For loop that is looking for empty clients.
          break;
        }   
      }
    }
  }

  return (1);
}

void addClientNum(char *theMessage, int clientNum)
{
  char returnBuffer[Max_Buffer];
	sprintf(returnBuffer, "%d#", clientNum);
	strcat(returnBuffer, theMessage);
	strcpy(theMessage, returnBuffer);
}

//Send the message to everyone
void Broadcast_Message(char* buffer, int size, int socketFrom)
{
	addClientNum(buffer, socketFrom);
	//printf("Plus ClientNum: %s", buffer);
	for(int i = 0; i < Max_Connections; i++)
	{
		if(Client[i].InUse == YES && i != socketFrom)
		//if(Client[i].InUse == YES)
		{
			int sendVal = send(Client[i].ClientSocket, buffer, strlen(buffer), 0);
      if(sendVal == SOCKET_ERROR)
			{
				printf("Error in Broadcast, disconnecting...");
        Disconnect(i);
			}
		}
	}
}



// Get the message, and see what it is.
void Get_Message(int s)
{
  // Receive The Data and Put into a Buffer Until we know what it is.
	const char *recvBuffer = readLine(Client[s].ClientSocket);
	char buffer[Max_Buffer];

	if (recvBuffer == NULL) return;

	//printf("Buffer = %s\n", recvBuffer);

  // Store the Ip of the Client that's sending message
  char *ClientIp = inet_ntoa(Client[s].clnt_addr.sin_addr);

  switch (recvBuffer[0])
  {
	case 'a':
		strcpy(buffer, recvBuffer);
		//printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 'b':
    strcpy(buffer, recvBuffer);
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 'c':
    //not currently used, used for server side polling for collisions
		break;
	case 'd':
		strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 'e':
		//not currently used, used to end a game but there is no world state to reset
		break;
	case 'g':
		//not currently used, used to start a game but there is no world state to initialize
		break;
	case 'i':
	{
		strcpy(buffer, recvBuffer);
    //store ship and team data
		Message::ParseInit(buffer, &Client[s].ship, &Client[s].team, &Client[s].pos);

		//printf("Before youMessage: %s\n", buffer);
    //send the player his player number
		char theYouMessage[Max_Buffer];
		Message::YouMessage(theYouMessage, s);
		int sendVal = send(Client[s].ClientSocket, theYouMessage, strlen(theYouMessage), 0);
    if(sendVal == SOCKET_ERROR)
		{
			printf("Error sending you message");
      Disconnect(s);
		}

    for( int i = 0; i < Max_Connections; i++ )
		{
			if( Client[i].InUse == YES && i != s )
			{
        //create an initMessage
				char initBuffer[Max_Buffer];
				Message::PlayerInit(initBuffer, Client[i].ship, Client[i].team, Client[i].pos);
				printf("Sending init data for Client %d: %s %d\n", i, initBuffer, strlen(initBuffer));
				addClientNum(initBuffer, i);

				//send to the joining player
				int sendVal = send(Client[s].ClientSocket, initBuffer, strlen(initBuffer), 0);
				if(sendVal == SOCKET_ERROR)
				{
					printf("Error sending new player init data\n");
					Disconnect(s);
				}
			}
		}

    //broadcast the other info to everyone else
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
    break;
	}
	case 'k':
    strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 'n':
		Disconnect(s);
		break;
  case 'p':
	  //append player number to begining of string
		strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		//printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
	  break;
	case 'q':
		Disconnect(s);
		break;
	case 'r':
		strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 's':
		//not currently used, the shield data isn't transmitted to other players
		break;
	case 'u':
		strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
	case 'v':
		strcpy(buffer, recvBuffer);
    //strcat(buffer, recvBuffer);
		//send it to the other clients
		printf("Broadcasting: %s %d", buffer, strlen(buffer));
		Broadcast_Message(buffer, strlen(buffer), s);
		break;
  default:
		cout << "Invalid Message Type: " << *recvBuffer << endl;
  }

	delete [] recvBuffer;
} 

// See if there is a Message coming in
void Check_Message()
{
  fd_set input_set, output_set, exc_set; // Create Input and Error sets for the select function
  int s, // This is Used for the Select command to store the value it gets back
  nfds; // Used for Compatability

  //timeout.tv_sec=0; // Used for Timeout
	timeout.tv_sec=1;
  timeout.tv_usec=0;

  //Set up the input,and exception sets for select().
  FD_ZERO(&input_set);
	FD_ZERO(&output_set);
  FD_ZERO(&exc_set);
  nfds = 0;

  // Loop makes it so that the Input and Error sets will look at all the Sockets connected
  for (int i = 0; i < Max_Connections; i++)
  {
		if (Client[i].InUse == YES)
		{
			FD_SET(Client[i].ClientSocket,&input_set);
			FD_SET(Client[i].ClientSocket,&output_set);
			FD_SET(Client[i].ClientSocket,&exc_set);
			nfds++;
		}
  }

  // The Actual Select Function sees if data coming in and stores it in s
  s = select(nfds,&input_set,&output_set,&exc_set,&timeout);
  if (s > 0) // Is there data coming in?
  {
    // Look Through all the clients to see which one is giving error or data
    for (int i=0; i < ClientsConnected;)
    {
      if (FD_ISSET(Client[i].ClientSocket,&exc_set)) // There was an Error?
      {
        // For this Server we assume it was a disconnection or something so close the socket.
        // Store the IP of the Client that Just Disconnected
        char *ClientIp = inet_ntoa(Client[i].clnt_addr.sin_addr);
        cout << "This Client Has Disconnected: " << ClientIp << endl;
        Disconnect(i);
      }

      if ((FD_ISSET(Client[i].ClientSocket, &input_set))) // Actual Data coming through?
      {
        // Get whatever message the client sending, and carry out appropriate response
        Get_Message(i);
      }
     i++; // Next Client
    }
  }
	else if (s == SOCKET_ERROR)
	{
		int ErrorCode = WSAGetLastError();
		if((ErrorCode != 10038) && //printf("Error Code: %d\n", ErrorCode);
		   (ErrorCode != 10022)) printf("Error Code: %d\n", ErrorCode);
	}
}

//reads a line terminated by newline character code adapted from Johnnie Rose Jr's winsock tutorial
char *readLine(SOCKET s) {
	vector<char> theVector;
  char buffer;
  int bytesReceived;

  while (true) {
    bytesReceived = recv(s, &buffer, 1, 0);

		if (bytesReceived <= 0)
      return "null";

    if (buffer == '\n') 
		{
			//don't forget we need to store the endline character
			theVector.push_back(buffer);
      char *pChar = new char[theVector.size() + 1];
      memset(pChar, 0, theVector.size() + 1);

      for (unsigned int f = 0; f < theVector.size(); f++) 
			{
        pChar[f] = theVector[f];
			}
			//printf("Reading Line: %s", pChar);
      return pChar;
    }
		else 
		{
      theVector.push_back(buffer);
    }
  }
}
// Main--------------------------------------------------------------------------------------------
int main()
{
  // Local Variables
  bool bye = false;

  // Initlizie Everything
  Setup();

  // This is a Continious loop aslong as bye == false, if not then it quits.
  while (bye == false)
  {
    // Check if any clients need connecting to server.
    Check_Connections();

    // Check to see if any messages are being sent from the clients.
    Check_Message();
  }	

  // Shutdown any Resources Used.
  Shutdown_Server();

  return (1); // Return Success
}