#include "Weapon.h"
#include "Player.h"
#include "Projectile.h"
#include "texture_loading.h"

//#define MULTI

#ifdef MULTI
#include "Message.h"
#include "ClientSocket.h"
extern ClientSocket connection;
#endif

Weapon::Weapon(const Player* pl, const Vector3f& lLoc, const Vector3f& rLoc, int type)
{
	player = pl;
	leftLocation = lLoc;
	rightLocation = rLoc;
	setType(type);
}

void Weapon::setType(int type)
{
	switch (type)
	{
		case PLASMA:
		{
			energyDrain = 10.0f;
			bulletPower = 10.0f;
			bulletSpeed = 2.0f;
			bulletSize = 0.2f;
			bulletTexture = TextureLoading::LoadTexture("Images/plasma_ball.tga");
			break;
		}
		case ENERGY:
		{
			energyDrain = 2.0f;
			bulletPower = 2.0f;
			bulletSpeed = 10.0f;
			bulletSize = 0.1f;
			bulletTexture = TextureLoading::LoadTexture("Images/energy_ball.tga");
			break;
		}
		case BLUE:
		{
			energyDrain = 4.0f;
			bulletPower = 4.0f;
			bulletSpeed = 8.0f;
			bulletSize = 0.1f;
			bulletTexture = TextureLoading::LoadTexture("Images/blue_ball.tga");
			break;
		}
		case LIGHTNING:
		{
			energyDrain = 6.0f;
			bulletPower = 6.0f;
			bulletSpeed = 6.0f;
			bulletSize = 0.2f;
			bulletTexture = TextureLoading::LoadTexture("Images/lightning_ball.tga");
			break;
		}
		case COOL:
		{
			energyDrain = 8.0f;
			bulletPower = 8.0f;
			bulletSpeed = 4.0f;
			bulletSize = 0.2f;
			bulletTexture = TextureLoading::LoadTexture("Images/cool_ball.tga");
			break;
		}
		// Add more weapons here
	}

	this->type = type;
}

void Weapon::fireOther() const
{
	Vector3f startPos = player->getPosition() + leftLocation;
	Projectile *lBullet = new Projectile(bulletTexture, startPos, bulletSize, player);
	lBullet->setPower(bulletPower);
	lBullet->setVelocity(player->getDirection() * (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	Projectile::addProj(lBullet);

	startPos.set(player->getPosition() + rightLocation);
	Projectile *rBullet = new Projectile(bulletTexture, startPos, bulletSize, player);
	rBullet->setPower(bulletPower);
	rBullet->setVelocity(player->getDirection() * (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	Projectile::addProj(rBullet);
}

void Weapon::fire() const
{
	Vector3f startPos = player->getPosition() + leftLocation;
	Projectile *lBullet = new Projectile(bulletTexture, startPos, bulletSize, player);
	lBullet->setPower(bulletPower);
	lBullet->setVelocity(player->getDirection() * (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	Projectile::addProj(lBullet);
	
	#ifdef MULTI
	char buffer[Max_Buffer];
	Message::BulletMessage(buffer, type, startPos, player->getDirection(), (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	connection.sendData(buffer, (int)  strlen(buffer));
	#endif
	
	startPos.set(player->getPosition() + rightLocation);
	Projectile *rBullet = new Projectile(bulletTexture, startPos, bulletSize, player);
	rBullet->setPower(bulletPower);
	rBullet->setVelocity(player->getDirection() * (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	Projectile::addProj(rBullet);

	#ifdef MULTI
	Message::BulletMessage(buffer, type, startPos, player->getDirection(), (float)(fabs(player->getSpeed())+1) * bulletSpeed);
	connection.sendData(buffer, (int) strlen(buffer));
	#endif
}