#include "hud.h"
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include "texture_loading.h"
#include "gl_font.h"
#include "LocalPlayer.h"

extern int windowWidth;
extern int windowHeight;
extern float last_frame;
extern float startTime;
extern float currentTime;
extern Camera *camera;
extern LocalPlayer* player;
extern ObjectCollection *objects;


unsigned int HUD::hud_texture = 0;
GLFont* HUD::hud_font1 = 0;
GLFont* HUD::hud_font2 = 0;
Vector3f HUD::pos;
Vector3f HUD::view;
float HUD::cos_angle;
const float HUD::RADAR_ANGLE = 3.14159f / 6.0f;


void HUD::Init() {

   hud_texture = TextureLoading::LoadTexture("Images/hud1.tga");
   //glDisable(GL_TEXTURE_2D);
   hud_font1 = new GLFont("Comic Sans MS", 20);
   hud_font2 = new GLFont("Comic Sans MS Bold", 26);

   cos_angle = cos(RADAR_ANGLE);
}

void HUD::Draw()
{
	glDisable(GL_DEPTH_TEST);
   glDisable(GL_LIGHTING);
   glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, windowWidth, 0, windowHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
   glLoadIdentity();

   // Background texture.
	glBindTexture(GL_TEXTURE_2D, hud_texture);
	glBegin(GL_QUADS);
	glColor3f(1.0f, 1.0f, 1.0f);
	glTexCoord2f(0.0f, 1.0f);
	glVertex2i(0, 0);
	glTexCoord2f(1.0f, 1.0f);
	glVertex2i(windowWidth, 0);
	glTexCoord2f(1.0f, 0.0f);
	glVertex2i(windowWidth, 70);
	glTexCoord2f(0.0f, 0.0f);
	glVertex2i(0, 70);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);

   glColor3f(1.0f, 0.05f, 0.05f);
   hud_font1->Printf(10.0f, 10.0f, "Frames per second: %d", (int)((1.0f / last_frame) + 0.5f));
	hud_font1->Printf(10.0f, 40.0f, "Current time: %2.2f", (currentTime - startTime)/1000.0f);

   // Draw the words.
   glColor3f(0.6f, 0.6f, 0.6f);
   hud_font1->Printf(350.0f, 5.0f, "SPEED");
   hud_font1->Printf(650.0f, 5.0f, "SHIELD");
   hud_font1->Printf(950.0f, 5.0f, "ENERGY");


   float color = player->getSpeed() / 10.0f;
   if (color > 1.0f)
      color = 1.0f;
   if (color < 0.0f)
      color = 0.0f;
   glColor3f(1.0f - color, color, 0.05f);
	hud_font2->Printf(363.0f, 25.0f, "%2.0f", player->getActualSpeed() * 10.0f);
	hud_font2->Printf(363.0f, 45.0f, "%2.0f", player->getSpeed() * 10.0f);

   color = player->getShield() / 100.0f;
   if (color > 1.0f)
      color = 1.0f;
   if (color < 0.0f)
      color = 0.0f;
   glColor3f(1.0f - color, color, 0.05f);
	hud_font2->Printf(665.0f, 25.0f, "%2.0f", player->getShield());

   color = player->getEnergy() / 100.0f;
   if (color > 1.0f)
      color = 1.0f;
   if (color < 0.0f)
      color = 0.0f;
   glColor3f(1.0f - color, color, 0.05f);
	hud_font2->Printf(965.0f, 25.0f, "%2.0f", player->getEnergy());


	//glColor3f(1.0f, 0.1f, 0.1f);
	//Menus::font->Printi(left + 20, bottom + 20, name);



	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

   glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);

   //DrawRadar();



   //DrawRadarDebugInfo();
}

void HUD::DrawRadar() {
   /*
   float modelview[16];
   glGetFloatv(GL_MODELVIEW_MATRIX, modelview);
   float projection[16];
   glGetFloatv(GL_PROJECTION_MATRIX, projection);


   glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, windowWidth, 0, windowHeight, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
*/

   //glLoadIdentity();

   // Get normalized view vector.
   pos = player->getPosition();
   //view = pos + player->rotAxis.z;
   //view = player->rotAxis.z;
   view = player->getDirection();
   view.normalize();

   // For every object.
   ObjectIter itr = objects->begin();
   while (itr != objects->end()) {
		Object3d* obj = *itr;

      Vector3f obj_pos = obj->getPosition();
      Vector3f obj_vec = obj_pos - pos;

      if (obj_vec.length() < 0.0001f) {
         itr++;
         continue;
      }
      obj_vec.normalize();

      // Take dot product.
      float dp = dotProduct(view, obj_vec);
      if (dp > cos_angle) {

         /*
         float transformed1[3];
         transformed1[0] = (modelview[0] * obj_pos.x) + (modelview[4] * obj_pos.y) +
            (modelview[8] * obj_pos.z);
         transformed1[1] = (modelview[1] * obj_pos.x) + (modelview[5] * obj_pos.y) +
            (modelview[9] * obj_pos.z);
         transformed1[2] = (modelview[2] * obj_pos.x) + (modelview[6] * obj_pos.y) +
            (modelview[10] * obj_pos.z);

         float transformed2[3];
         transformed2[0] = (projection[0] * transformed1[0]) + (projection[4] * transformed1[1]) +
            (projection[8] * transformed1[2]);
         transformed2[1] = (projection[1] * transformed1[0]) + (projection[5] * transformed1[1]) +
            (projection[9] * transformed1[2]);
         transformed2[2] = (projection[2] * transformed1[0]) + (projection[6] * transformed1[1]) +
            (projection[10] * transformed1[2]);
         */

         glBegin(GL_LINES);
         glColor3f(0.0f, 1.0f, 0.0f);
         //glVertex2f(700.0f, 500.0f);
         //glVertex2f(700.0f + 10.0f * obj_pos.x, 500.0f + 10.0f * obj_pos.y);
         glVertex3f(pos.x + view.x * 10.0f,
            pos.y + view.y * 10.0f, pos.z + view.z * 10.0f);
         glVertex3f(obj_pos.x, obj_pos.y, obj_pos.z);
         glEnd();

         // Project the vector onto the screen plane.
         //Vector3f projected;
         //Vector3f temp;
         //crossProduct(obj_vec, view, &temp);
         //crossProduct(temp, view, &projected);

      }

		itr++;
	}
/*
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
   */
}

/*void HUD::DrawRadarDebugInfo() {


}*/
