#include "Matrix.h"
#include "Quaternion.h"

// There must be some easier (and/or faster) way to do this
Matrix operator*(const Matrix& A, const Matrix& B)
{
	return Matrix(// First column
				  A.m11 * B.m11 + A.m12 * B.m21 + A.m13 * B.m31 + A.m14 * B.m41,
				  A.m21 * B.m11 + A.m22 * B.m21 + A.m23 * B.m31 + A.m24 * B.m41,
				  A.m31 * B.m11 + A.m32 * B.m21 + A.m33 * B.m31 + A.m34 * B.m41,
				  A.m41 * B.m11 + A.m42 * B.m21 + A.m43 * B.m31 + A.m44 * B.m41,
				  // Second column
				  A.m11 * B.m12 + A.m12 * B.m22 + A.m13 * B.m32 + A.m14 * B.m42,
				  A.m21 * B.m12 + A.m22 * B.m22 + A.m23 * B.m32 + A.m24 * B.m42,
				  A.m31 * B.m12 + A.m32 * B.m22 + A.m33 * B.m32 + A.m34 * B.m42,
				  A.m41 * B.m12 + A.m42 * B.m22 + A.m43 * B.m32 + A.m44 * B.m42,
				  // Third column
				  A.m11 * B.m13 + A.m12 * B.m23 + A.m13 * B.m33 + A.m14 * B.m43,
				  A.m21 * B.m13 + A.m22 * B.m23 + A.m23 * B.m33 + A.m24 * B.m43,
				  A.m31 * B.m13 + A.m32 * B.m23 + A.m33 * B.m33 + A.m34 * B.m43,
				  A.m41 * B.m13 + A.m42 * B.m23 + A.m43 * B.m33 + A.m44 * B.m43,
				  // Fourth column
				  A.m11 * B.m14 + A.m12 * B.m24 + A.m13 * B.m34 + A.m14 * B.m44,
				  A.m21 * B.m14 + A.m22 * B.m24 + A.m23 * B.m34 + A.m24 * B.m44,
				  A.m31 * B.m14 + A.m32 * B.m24 + A.m33 * B.m34 + A.m34 * B.m44,
				  A.m41 * B.m14 + A.m42 * B.m24 + A.m43 * B.m34 + A.m44 * B.m44
				 );
}

Quaternion operator*(const Quaternion& q1, const Quaternion& q2)
{
	float A, B, C, D, E, F, G, H;

	A = (q1.w + q1.x)*(q2.w + q2.x);
	B = (q1.z - q1.y)*(q2.y - q2.z);
	C = (q1.w - q1.x)*(q2.y + q2.z); 
	D = (q1.y + q1.z)*(q2.w - q2.x);
	E = (q1.x + q1.z)*(q2.x + q2.y);
	F = (q1.x - q1.z)*(q2.x - q2.y);
	G = (q1.w + q1.y)*(q2.w - q2.z);
	H = (q1.w - q1.y)*(q2.w + q2.z);

	float x = A - (E + F + G + H) / 2.0f; 
	float y = C + (E - F + G - H) / 2.0f; 
	float z = D + (E - F - G + H) / 2.0f;
	float w = B + (-E - F + G + H) / 2.0f;
/*	
	result.x = A.w*B.x + A.x*B.w + A.y*B.z - A.z*B.y;
	result.y = A.w*B.y - A.x*B.z + A.y*B.w + A.z*B.x;
	result.z = A.w*B.z + A.x*B.y - A.y*B.x + A.z*B.w;
	result.w = A.w*B.w - A.x*B.x - A.y*B.y - A.z*B.z;
*/
	return Quaternion(x, y, z, w);
}

// (Copied from www.gametutorials.com)
// This function takes a line segment, vA to vB, then a point, vPoint.
// We want to find the closet point on the line segment to our vPoint
// out in space.  Either it is going to be one of the end points of the line,
// or it is going to be somewhere between vA and vB.  
Vector3f closestPointOnLine(const Vector3f& vA, const Vector3f& vB, const Vector3f& vPoint)
{
	// Create the vector from end point vA to our point vPoint.
	Vector3f vVector1 = vPoint - vA;

	// Create a normalized direction vector from end point vA to end point vB
    Vector3f vVector2 = vB - vA;
	if (equals(vVector2.length(), 0.0f)) return vA;
	vVector2.normalize();

	// Use the distance formula to find the distance of the line segment (or magnitude)
    float d = vA.distance(vB);

	// Using the dot product, we project the vVector1 onto the vector vVector2.
	// This essentially gives us the distance from our projected vector from vA.
    float t = dotProduct(vVector2, vVector1);

	// If our projected distance from vA, "t", is less than or equal to 0, it must
	// be closest to the end point vA.  We want to return this end point.
    if (t <= 0) 
		return vA;

	// If our projected distance from vA, "t", is greater than or equal to the magnitude
	// or distance of the line segment, it must be closest to the end point vB.  So, return vB.
    if (t >= d) 
		return vB;
 
	// Here we create a vector that is of length t and in the direction of vVector2
    Vector3f vVector3 = vVector2 * t;

	// To find the closest point on the line segment, we just add vVector3 to the original
	// end point vA.  
    Vector3f vClosestPoint = vA + vVector3;

	// Return the closest point on the line segment
	return vClosestPoint;
}

// Float equality (epsilon = 0.0005)
bool equals(float x, float y)
{ return (fabs(x-y) < 0.0005f); }

// Returns a random number in the interval (0, range)
float random(float range)
{ return ((rand() / (float) RAND_MAX) * range); }

// Returns a random number in the interval (min, max)
float random(float min, float max)
{ return (random(max-min) + min); }

// Returns a random vector in the interval (min, max)
Vector3f randomVector(float min, float max)
{ return Vector3f(random(min, max), random(min, max), random(min, max)); }