#ifndef MATH_UTIL_H
#define MATH_UTIL_H

#include "Vector3.h"

template <class Type>
Type bound(Type min, Type x, Type max)
{ return (x < min ? min : max < x ? max : x); }

bool equals(float x, float y);

struct Matrix;
Matrix operator*(const Matrix& A, const Matrix& B);

struct Quaternion;
Quaternion operator*(const Quaternion& A, const Quaternion& B);

Vector3f closestPointOnLine(const Vector3f& vA, const Vector3f& vB, const Vector3f& vPoint);

// Float equality (epsilon = 0.0005)
bool equals(float x, float y);

// Returns a random number in the interval (0, range)
float random(float range);

// Returns a random number in the interval (min, max)
float random(float min, float max);

// Returns a random vector in the interval (min, max)
Vector3f randomVector(float min, float max);

const float PI = 3.141592654f;
const Vector3f UpVector(0.0f, 1.0f, 0.0f);

#endif
