#include "skybox.h"
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <assert.h>
#include "texture_loading.h"
#include "Camera.h"

extern Camera *camera;


unsigned int Skybox::skybox_front = 0;
unsigned int Skybox::skybox_back = 0;
unsigned int Skybox::skybox_left = 0;
unsigned int Skybox::skybox_right = 0;
unsigned int Skybox::skybox_top = 0;
unsigned int Skybox::skybox_bottom = 0;
unsigned int Skybox::skybox_display_list = 0;


void Skybox::Init() {
   
   float dim = 10.0f;

   skybox_front = TextureLoading::LoadTexture("Images/sky_ft.tga");
   assert(skybox_front);
   skybox_back = TextureLoading::LoadTexture("Images/sky_bk.tga");
   assert(skybox_back);
   skybox_left = TextureLoading::LoadTexture("Images/sky_lf.tga");
   assert(skybox_left);
   skybox_right = TextureLoading::LoadTexture("Images/sky_rt.tga");
   assert(skybox_right);
   skybox_top = TextureLoading::LoadTexture("Images/sky_up.tga");
   assert(skybox_top);
   skybox_bottom = TextureLoading::LoadTexture("Images/sky_dn.tga");
   assert(skybox_bottom);

   skybox_display_list = glGenLists(1);
   glNewList(skybox_display_list, GL_COMPILE);

   glEnable(GL_TEXTURE_2D);

   //glDisable(GL_FOG);	
   glBindTexture(GL_TEXTURE_2D, skybox_back);
	glBegin(GL_QUADS);		
		
		// Back.
		glTexCoord2f(0.999f, 0.001f); glVertex3f(dim, -(dim / 2.0f), -dim);
		glTexCoord2f(0.999f, 0.999f); glVertex3f(dim, dim / 2.0f, -dim); 
		glTexCoord2f(0.001f, 0.999f); glVertex3f(-dim, dim / 2.0f, -dim);
		glTexCoord2f(0.001f, 0.001f); glVertex3f(-dim, -(dim / 2.0f), -dim);
	glEnd();


   glBindTexture(GL_TEXTURE_2D, skybox_front);
	glBegin(GL_QUADS);	
	
		// Front.
		glTexCoord2f(0.999f, 0.001f); glVertex3f(-dim, -(dim / 2.0f), dim);
		glTexCoord2f(0.999f, 0.999f); glVertex3f(-dim, dim / 2.0f, dim);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(dim, dim / 2.0f, dim); 
		glTexCoord2f(0.001f, 0.001f); glVertex3f(dim, -(dim / 2.0f), dim);
	glEnd();


   glBindTexture(GL_TEXTURE_2D, skybox_bottom);
	glBegin(GL_QUADS);		
	
		// Bottom
		glTexCoord2f(0.999f, 0.001f); glVertex3f(-dim, -(dim / 2.0f), -dim);
		glTexCoord2f(0.999f, 0.999f); glVertex3f(-dim, -(dim / 2.0f), dim);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(dim, -(dim / 2.0f), dim); 
		glTexCoord2f(0.001f, 0.001f); glVertex3f(dim, -(dim / 2.0f), -dim);
	glEnd();


   glBindTexture(GL_TEXTURE_2D, skybox_top);
	glBegin(GL_QUADS);		
		
		// Top.
		glTexCoord2f(0.001f, 0.001f); glVertex3f(dim, dim / 2.0f, -dim);
		glTexCoord2f(0.999f, 0.001f); glVertex3f(dim, dim / 2.0f, dim); 
		glTexCoord2f(0.999f, 0.999f); glVertex3f(-dim, dim / 2.0f, dim);
		glTexCoord2f(0.001f, 0.999f); glVertex3f(-dim, dim / 2.0f, -dim);
	glEnd();


   glBindTexture(GL_TEXTURE_2D, skybox_left);
	glBegin(GL_QUADS);		
		
		// Left.
		glTexCoord2f(0.999f, 0.999f); glVertex3f(-dim, dim / 2.0f, -dim);	
		glTexCoord2f(0.001f, 0.999f); glVertex3f(-dim, dim / 2.0f, dim); 
		glTexCoord2f(0.001f, 0.001f); glVertex3f(-dim, -(dim / 2.0f), dim);
		glTexCoord2f(0.999f, 0.001f); glVertex3f(-dim, -(dim / 2.0f), -dim);		
	glEnd();


   glBindTexture(GL_TEXTURE_2D, skybox_right);
	glBegin(GL_QUADS);		

		// Right.
		glTexCoord2f(0.001f, 0.001f); glVertex3f(dim, -(dim / 2.0f), -dim);
		glTexCoord2f(0.999f, 0.001f); glVertex3f(dim, -(dim / 2.0f), dim);
		glTexCoord2f(0.999f, 0.999f); glVertex3f(dim, dim / 2.0f, dim); 
		glTexCoord2f(0.001f, 0.999f); glVertex3f(dim, dim / 2.0f, -dim);
	glEnd();
   glBindTexture(GL_TEXTURE_2D, 0);

   //glEnable(GL_FOG);	

   glEndList();
}

void Skybox::Draw() {

   glPushMatrix();
   glTranslatef(camera->position.x, camera->position.y, camera->position.z);
   //glPushAttrib(GL_COLOR_BUFFER_BIT);
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_LIGHTING);
   //glDisable(GL_COLOR_MATERIAL);
   glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
   glCallList(skybox_display_list);
   //glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_LIGHTING);
   glEnable(GL_DEPTH_TEST);
   //glPopAttrib();
   glPopMatrix();
   //glColor3f(1.0f, 1.0f, 1.0f);
}
