// Remove this for non-windows compiles
#include <windows.h>

#include <cstdio>
#include <cstdlib>
#include <GL/glut.h>
#include <time.h>

// If you don't have the CG .h & .lib files, comment out this line and its identical on Player.cpp
// Also you might want to remove Shader.cpp and PlayerShield.cpp from the project
#define USE_SHADERS

// If you don't want to have multiplayer behavior, comment out this line 
// from here, Player.cpp and Weapon.cpp
//#define MULTI
#define SERVER_IP "129.65.51.50"

#include "math_util.h"
#include "Camera.h"
#include "FrameCount.h"
#include "Mesh.h"
#include "Object3d.h"
#include "Frustrum.h"
#include "Player.h"
#include "LocalPlayer.h"
#include "Projectile.h"
#include "BoundingSphere.h"
#include "menus.h"
#include "hud.h"
#include "skybox.h"
#include "Spacecraft.h"
#include "Object2d.h"
#include "Weapon.h"

#ifdef MULTI
#include "Message.h"
#include "ClientSocket.h"
#else
#define Max_Buffer 512
#endif

#ifdef USE_SHADERS
#include "Shader.h"
#endif

using namespace std;

#define MAX_SPACECRAFTS 10

const Vector3f StartVector(1.0f, 0.0f, 0.0f);

Vector3f teamColors[4] = 
{ Vector3f(1.0f, 0.0f, 0.0f), Vector3f(0.0f, 0.0f, 1.0f),
Vector3f(0.0f, 1.0f, 0.0f), Vector3f(1.0f, 1.0f, 1.0f) };

/* The text that appears on the top left part of the screen */
const float CharHeight = 0.025f;
const Vector3f textColor(0.0f, 0.0f, 1.0f);
const Vector3f StatsPosition(0.05f, 0.95f, 0.0f);
const int TextLines = 7;
char stats[TextLines][50];

// Time counting
float startTime, currentTime, previousTime;
float last_frame;

// Viewport global variables
int windowWidth = GetSystemMetrics(SM_CXSCREEN);
int windowHeight = GetSystemMetrics(SM_CYSCREEN);
float zNearClip = 0.1f;
float zFarClip = 200.0f;

// Mouse control variables
int startX = windowWidth/2, startY = windowHeight/2;
int endx, endy;
int pressed = -1;

const Vector3f ZeroVector(0.0f, 0.0f, 0.0f);

// Important stuff
Camera *camera;
Frustrum frustrum;
ObjectCollection *objects = &Object3d::objects;
map<int, Player*> players;
int myNum;
LocalPlayer *player;
Weapon *gWeapon;
Spacecraft *spacecrafts[MAX_SPACECRAFTS];
int num_spacecrafts;

#ifdef MULTI
// The client
ClientSocket connection(SERVER_IP, 12345);
#endif

bool menu_active = true;

void loadSpacecrafts(char *filename) 
{
   char strMessage[Max_Buffer] = {0};
   char buffer[Max_Buffer];
   char currentModelFilename[Max_Buffer];
   float angle, l_x, l_y, l_z, r_x, r_y, r_z;
   FILE *fileptr = fopen(filename, "r");

	// Make sure we have a valid file pointer (we found the file)
	if (!fileptr)
	{
		fprintf(stderr, "Unable to find the file: %s\n", filename);
		exit(-1);
	}
   
   num_spacecrafts = 0;
   unsigned int metal_tex = TextureLoading::LoadTexture("Images/titanium.tga");
   while(fgets (buffer , Max_Buffer , fileptr) && num_spacecrafts < MAX_SPACECRAFTS)
   {
      sscanf(buffer,"%s %f %f %f %f %f %f %f",&currentModelFilename, &angle, &l_x, &l_y, &l_z, &r_x, &r_y, &r_z);
      Vector3f left_weapon, right_weapon;
      left_weapon.set(l_x, l_y, l_z);
      right_weapon.set(r_x, r_y, r_z);
      spacecrafts[num_spacecrafts] = new Spacecraft(currentModelFilename, angle, 1.0f, metal_tex, left_weapon, right_weapon);
      num_spacecrafts++;      
   }
   fclose(fileptr);
}

void initGame()
{
	srand(GetTickCount());

	cout << "Loading the spacecraft models" << endl;
	loadSpacecrafts("Data/craft_info.dat");

#ifdef USE_SHADERS
	Shader::Init();
#endif

	Spacecraft *playerShip = spacecrafts[0];
	// Avoid starting the player with a direction.y different than 0.0
	player = new LocalPlayer(playerShip, Vector3f(-60.0f, 0.0f, 0.0f), Vector3f(1.0f, 0.0f, 0.0f));
	player->setColor(teamColors[0]);

	int size;
	Model3ds *asteroid;
	unsigned int ast_tex;
	unsigned int asteroid_tex1 = TextureLoading::LoadTexture("Images/ast01.tga");
	unsigned int asteroid_tex2 = TextureLoading::LoadTexture("Images/ast02.tga");
	unsigned int asteroid_tex3 = TextureLoading::LoadTexture("Images/ast03.tga");
	ifstream asteroidFile("Data/asteroids.dat");
	char buffer[Max_Buffer];

	for(int i = 0; i < 7; i++)
	{
		size = (rand() % 8) + 1;
		ast_tex = (rand() % 3) + 1;

		//random selection between 3 asteroid textures
		if(ast_tex == 1)
			ast_tex = asteroid_tex1;
		else if(ast_tex == 2)
			ast_tex = asteroid_tex2;
		else
			ast_tex = asteroid_tex3;

		Vector3f asteroidColor(0.36f, 0.34f, 0.28f);
		asteroid = new Model3ds("Models/Asteroid_1.3ds", 0.0f, (float)size, ast_tex);
		
		for (int i = 0; i < 63; i++)
		{
			asteroidFile.getline(buffer, Max_Buffer);
			float vx, vy, vz, qx, qy, qz, qw;
			int result = sscanf(buffer, "(%f, %f, %f) (%f, %f, %f, %f)\n", &vx, &vy, &vz, &qx, &qy, &qz, &qw);

			assert (result == 7);
            
			Object3d *object = new Object3d(asteroid, Vector3f(vx, vy, vz));
			object->setRotation(Quaternion(qx, qy, qz, qw));
			object->setColor(asteroidColor);
		}
	}

	camera = new Camera(Vector3f(0.0f, 0.0f, 0.0f), player);
	player->setCamera(camera);

	Skybox::Init();
	HUD::Init();
	Menus::Init();

	startTime = (float) GetTickCount();
	previousTime = currentTime = startTime;
}

void playerInit() 
{
	delete player;
	delete camera;
	Model3ds *playerShip = spacecrafts[Menus::ship_type];
	Vector3f startPosition = randomVector(-50.0f, 50.0f);
	startPosition.x = (startPosition.x / 10.0f) - 55.0f;
	//Vector3f startDirection = Vector3f(1.0f, 0.0f, 0.0f);
	player = new LocalPlayer(playerShip, startPosition, StartVector);
	camera = new Camera(Vector3f(0.0f, 0.0f, 0.0f), player);
	player->setColor(teamColors[Menus::team]);
	player->setCamera(camera);
	player->setPrimaryWeapon(Menus::primary_weapon_type);
	player->setSecondaryWeapon(Menus::secondary_weapon_type);

	// Menus::team will contain the player's choice for team, integer 0-3.
	#ifdef MULTI
	char buffer[Max_Buffer];
	Message::PlayerInit(buffer, Menus::ship_type, Menus::team, startPosition);
	if (connection.sendData(buffer, (int) strlen(buffer)))
	{
		memset(buffer, 0, Max_Buffer);
		while (!connection.readLine(buffer));
		Message::ParseYou(buffer, &myNum);
	}
	#endif
}

//mouse event function
void mouse(int button, int state, int x, int y)
{
	if (menu_active)
	{
		if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_UP))
			Menus::MouseClick(x, windowHeight - y);
	}
	else if ((button == GLUT_LEFT_BUTTON) && (state == GLUT_DOWN))
		player->fireFirst();
	else if ((button == GLUT_RIGHT_BUTTON) && (state == GLUT_DOWN))
		player->fireSecond();
}

void keyboard2(int key, int x, int y)
{
   if (!menu_active) {
	   switch(key)
	   {
		   case GLUT_KEY_HOME:
		   {
			   ObjectIter objItr = objects->begin();
			   ObjectIter objEndItr = objects->end();
			   for (; objItr != objEndItr; objItr++)
				   (*objItr)->toggleBVolumeVisible();

			   break;
		   }
		   case GLUT_KEY_F1:
			   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			   break;
		   case GLUT_KEY_F2:
			   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			   break;
	   }

	   glutPostRedisplay();
   }
}

void Quit()
{
	#ifdef USE_SHADERS
	Shader::Deinitialize();
	#endif

	#ifdef MULTI
	char buffer[Max_Buffer];
	Message::PlayerQuit(buffer);
	connection.sendData(buffer, (int) strlen(buffer));
	#endif
	exit(0);
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key)
	{
		case 27: // ESC
		{
			Quit();
			break;
		}

		default:
         if (!menu_active)
			   player->keyboardInput(key);
	}

	glutPostRedisplay();
}

void updateObjects(float time)
{
   //if (menu_active)
   //   return;

	Projectile::updateAll(time);
	player->update(time);

	#ifdef MULTI
	char buffer[Max_Buffer];
	memset(buffer, 0, Max_Buffer);
	while (connection.readLine(buffer))
	{
		//cout << "got message " << buffer << endl;
		int key = Message::getClientNum(buffer);
		Message::StripClientNum(buffer);

		if (key == myNum) cout << "Something wrong here, I got a message with my own number as a client num" << endl;
        
		switch (*buffer)
		{
			case 'b':
			{
				int type; Vector3f pos, dir; float vel;
				Message::ParseBullet(buffer, &type, &pos, &dir, &vel);
				
				map<int, Player*>::iterator plIter = players.find(key);
				
				if (plIter != players.end())
					players[key]->fireOther(type);

				break;
			}
			case 'i':
			{
				int ship, team;
				Vector3f startPos;
				Message::ParseInit(buffer, &ship, &team, &startPos);
				players[key] = new Player(spacecrafts[ship], startPos, StartVector);
				players[key]->setColor(teamColors[team]);
				break;
			}
			case 'k':
			{
				players[key]->die();
				delete players[key];
				players.erase(key);
				break;
			}			
			case 'a':
			{
				map<int, Player*>::iterator plIter = players.find(key);
				
				if (plIter != players.end())
					(*plIter).second->updateFromServer(buffer);

				break;
			}
		}
	}
	#endif
	   
	map<int, Player*>::iterator plIter = players.begin();
	map<int, Player*>::iterator plEndIter = players.end();

	for (; plIter != plEndIter; plIter++)
		(*plIter).second->update(time);
	
	#ifdef MULTI
	memset(buffer, 0, Max_Buffer);
	Message::AllMessage(buffer, player->getPosition(), player->getDirection(), player->getUpvector(), player->getSpeed(), player->getRotation());
	connection.sendData(buffer, (int) strlen(buffer));
	#endif
}

void drawObjects()
{
	ObjectIter objItr = objects->begin();
	ObjectIter objEndItr = objects->end();
	for (; objItr != objEndItr; objItr++)
	{
		(*objItr)->testFrustrum(frustrum);
		(*objItr)->draw();
	}

   glBindTexture(GL_TEXTURE_2D, 0);
}

void drawAlpha()
{
	glDepthMask(GL_FALSE);

	ObjectIter objItr = objects->begin();
	ObjectIter objEndItr = objects->end();
	for (; objItr != objEndItr; objItr++)
		(*objItr)->drawBoundingVolume();
/*
	vector<Player*>::iterator plItr = players.begin();
	vector<Player*>::iterator plEndItr = players.end();
	for (; plItr != plEndItr; plItr++)
		(*plItr)->drawBoundingVolume();
*/
	Projectile::drawAll();

	//player->drawBoundingVolume();

	glDepthMask(GL_TRUE);
}

void reshape(int width, int height)
{
	windowWidth = width;
	windowHeight = height;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(80.0, (float) width / (float) height, zNearClip, zFarClip);

	glMatrixMode(GL_MODELVIEW);
	glViewport(0, 0, width, height);

	glutPostRedisplay();
}


void display()
{
	last_frame = frameEnd();
	frameStart();
	
	glClearDepth(1.0f);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	camera->apply();

	frustrum.recalcPlanes();

	Skybox::Draw();

	// Go through all the objects in the screen and test
	// their bounding volumes against the frustrum
	drawObjects();

	// Draw transparent stuff last.
	drawAlpha();

	player->drawCrossHair();
	//displayStats();
	HUD::Draw();

	glPopMatrix();

	if (menu_active)
		Menus::Draw();

	glutSwapBuffers();
}


void idle()
{
	if (!menu_active)
	{
		currentTime = (float) GetTickCount();
		float elapsedTime = (currentTime - previousTime) / 1000.0f;
		updateObjects(min(elapsedTime, 10.0f));
		previousTime = currentTime;

		POINT mousePos;
		GetCursorPos(&mousePos);					
		float deltaX = (float) (mousePos.x - startX) / (windowWidth*10.0f);
		float deltaY = (float) (mousePos.y - startY) / (windowHeight*10.0f);
		
		if (player->alive)
			player->mouseInput(-deltaX, -deltaY);

		
	}
   glutPostRedisplay();
}

int main( int argc, char** argv )
{
	//set up my window
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	//glutInitWindowSize(windowWidth, windowHeight);
	//glutInitWindowPosition(100, 100);
	glutCreateWindow("Insane Games");
	glutFullScreen();

	// set up the viewport
	reshape(windowWidth, windowHeight);

	//enable z-buffer
	glEnable(GL_DEPTH_TEST);

	//enable lighting and shading
	GLfloat lightPos[4] = { 0.0f, 10.0f, 0.0f, 1.0f };
	glLightfv(GL_LIGHT0,  GL_POSITION, lightPos);
	GLfloat lightAmb[4] = { 0.7, 0.7, 0.7, 1.0};
	glLightfv(GL_LIGHT0,  GL_AMBIENT, lightAmb);
	GLfloat lightDif[4] = { 0.5f, 0.5f, 0.5f, 1.0};
	glLightfv(GL_LIGHT0,  GL_DIFFUSE, lightDif);

	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_TEXTURE_2D);
	// Alpha blending.
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	//register glut callback functions
	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(keyboard2);
	glutMouseFunc(mouse);
	//glutPassiveMotionFunc(mouseMove);

	cout << "Initializing game..." << endl;
	initGame();

	frameStart();

	// Loop forever
	glutMainLoop();
}
