#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glaux.h>
#include "texture_loading.h"

std::map<std::string, unsigned int, ltstr> TextureLoading::textures;

unsigned int TextureLoading::LoadTexture(const char* filename)
{
	std::map<std::string, unsigned int, ltstr>::const_iterator mapIter;
	mapIter = textures.find(std::string(filename));

	if (mapIter == textures.end())
		return GetTexture(filename);
	else
		return (*mapIter).second;
}

unsigned int TextureLoading::GetTexture(const char* filename) {

   unsigned int texture;
   unsigned int width;
   unsigned int height;
   unsigned char tga_header[12] = {0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char compare[12];
	unsigned char header[6];
	unsigned int bpp;
   unsigned int bpp2;
	unsigned int size;
	unsigned char temp;
	unsigned int type = GL_RGBA;

   FILE* file = fopen(filename, "rb");

	if (file == NULL || fread(compare, 1, sizeof(compare), file) != sizeof(compare)
    || memcmp(tga_header, compare, sizeof(tga_header)) != 0 || fread(header, 1, sizeof(header), file)
    != sizeof(header)) {
		if (file == NULL)
		{
			fprintf(stderr, "Could not load texture file %s!!\n", filename);
			return NULL;
		}
		else 
		{
			fclose(file);
			fprintf(stderr, "Error in texture file %s!!\n", filename);
			return NULL;
		}
	}

	fprintf(stderr, "Texture file %s loaded succesfully\n", filename);

	width  = header[1] * 256 + header[0];
	height = header[3] * 256 + header[2];
    
 	if (width <= 0 || height <= 0 || (header[4] != 24 && header[4] != 32)) {
		fclose(file);
		return NULL;
	}

   bpp2 = header[4];
	bpp = bpp2 / 8;
	size = width * height * bpp;

   unsigned char* data = (unsigned char*)malloc(size);

	if (data == NULL || fread(data, 1, size, file) != size) {
		if (data != NULL)
			free(data);

		fclose(file);
		return NULL;
	}

   for (unsigned int i = 0; i < size; i += bpp) {
		temp = data[i];
		data[i] = data[i + 2];
		data[i + 2] = temp;
	}

	fclose(file);

   glGenTextures(1, (unsigned int*)&(texture));

	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

   if (bpp2 == 24)
		type = GL_RGB;

	glTexImage2D(GL_TEXTURE_2D, 0, type, width, height, 0, type,
    GL_UNSIGNED_BYTE, data);

	textures[std::string(filename)] = texture;

	return texture;	
}